from datetime import datetime

from django.core.exceptions import ValidationError
from django.contrib.auth.models import User
from django.db import models

from repos.services import (
    SERVICES,
    get_service_from_url,
)

STATUS_ERROR = 'error'
STATUS_MERGED = 'merged'
STATUS_PENDING = 'pending'

STATUS_CHOICES = [
    (STATUS_ERROR, 'Failed'),
    (STATUS_MERGED, 'Merged'),
    (STATUS_PENDING, 'Pending'),
]


def service_url_validator(value):
    if not any(getattr(s, 'is_valid_url')(value)
               for s in SERVICES.itervalues()):
        raise ValidationError('%s is not a valid service url' % value)


class Project(models.Model):
    """A project."""

    owner = models.ForeignKey(User)
    name = models.TextField()
    url = models.CharField(max_length=256, validators=[service_url_validator])

    @property
    def service(self):
        return get_service_from_url(self.url)

    def approved_requests(self):
        """Return approved merge proposals."""
        mrs = self.service.approved_requests_for_project_url(self.url)
        for mr in mrs:
            MergeRequest.objects.get_or_create(project=self, **mr)

        return MergeRequest.objects.filter(project=self, status=STATUS_PENDING)


class MergeRequest(models.Model):
    """Merge proposal abstraction."""

    project = models.ForeignKey(Project)
    status = models.CharField(choices=STATUS_CHOICES, max_length=128,
                              default=STATUS_PENDING)
    source = models.TextField()
    target = models.TextField()
    commit_message = models.TextField()
    author = models.TextField(default='')
    reviewers = models.TextField()
    date_created = models.DateTimeField()
    date_completed = models.DateTimeField(null=True, blank=True)

    def complete(self, error=False):
        self.date_completed = datetime.utcnow()
        self.status = STATUS_MERGED if not error else STATUS_ERROR
        self.save()


class RunLog(models.Model):

    merge_request = models.ForeignKey(MergeRequest)
    timestamp = models.DateTimeField(default=datetime.utcnow)
    raw = models.TextField()
    return_code = models.IntegerField()

    class Meta:
        get_latest_by = 'timestamp'
