from datetime import datetime, timedelta

from django.core.exceptions import ValidationError
from mock import patch

from locolanderweb.models import (
    STATUS_CHOICES,
    STATUS_ERROR,
    STATUS_MERGED,
    STATUS_PENDING,
    MergeRequest,
    Project,
    RunLog,
    service_url_validator,
)
from locolanderweb.tests.test_base import BaseTestCase

MERGE_REQUESTS = [
    dict(
        source='source1', target='target1', reviewers='a,b,c',
        date_created=datetime(2013, 4, 23), commit_message='something 1',
    ),
    dict(
        source='source2', target='target2', reviewers='pepe',
        date_created=datetime(2012, 2, 29), commit_message='something else',
    ),
    dict(
        source='source3', target='target3', reviewers='pepe y pepa',
        date_created=datetime.utcnow(), commit_message='',
    ),
]

INVALID_URLS = (
    'locolander',
    'launchpad.net',
    'launchpad.net/foo',
    ###'lp:launchpad.net/foo',
    'https://github.com/username',
    'git@github.com:foo/dippi-dapi',
)

VALID_URLS = (
    'http://launchpad.net/locolander',
    'http://launchpad.net/~pepe/locolander',
    'https://launchpad.net/locolander',
    'https://launchpad.net/~pepe/locolander',
    'lp:locolander',
    'lp:~pepe/locolander/trunk',
    'https://github.com/username/yadda-yoda.git',
    'git@github.com:foo/dippi-dapi.git',
)


class ServiceUrlValidatorTestCase(BaseTestCase):

    def test_invalid(self):
        for url in INVALID_URLS:
            with self.assertRaises(ValidationError) as ctx:
                service_url_validator(url)

            e = ctx.exception
            self.assertIn(url, unicode(e))

    def test_valid(self):
        for url in VALID_URLS:
            service_url_validator(url)


class ProjectTestCase(BaseTestCase):

    url = 'http://zaraza.foo'

    def setUp(self):
        super(ProjectTestCase, self).setUp()
        self.user = self.factory.make_user()
        self.obj = Project.objects.create(
            owner=self.user, name='foo', url=self.url)

    def test_owner(self):
        self.assertEqual(self.obj.owner, self.user)

    def test_url(self):
        for url in VALID_URLS:
            self.obj.url = url
            self.obj.full_clean()  # no error

        for url in INVALID_URLS:
            self.obj.url = url
            with self.assertRaises(ValidationError):
                self.obj.full_clean()

    def test_name(self):
        self.obj.name = expected = 'a' * 100
        self.obj.save()
        self.assertEqual(self.obj.name, expected)

    def test_service(self):
        self.assertIs(self.obj.service, self.get_service_from_url.return_value)
        self.get_service_from_url.assert_called_once_with(self.obj.url)

    def test_approved_requests(self):
        assert MergeRequest.objects.all().count() == 0

        service = self.get_service_from_url.return_value
        service.approved_requests_for_project_url.return_value = MERGE_REQUESTS

        result = self.obj.approved_requests()
        self.assertEqual(MergeRequest.objects.all().count(), len(result))
        for mr in MERGE_REQUESTS:
            self.assertEqual(
                MergeRequest.objects.filter(**mr).count(), 1)


class MergeRequestTestCase(BaseTestCase):

    def setUp(self):
        super(MergeRequestTestCase, self).setUp()
        self.project = self.factory.make_project()
        self.created = datetime(2012, 05, 10)
        self.obj = MergeRequest.objects.create(
            project=self.project,
            source='source', target='target', commit_message='commit',
            author='Some Person <some@domain>', reviewers='fulano, mengano',
            date_created=self.created)

    def test_status_default(self):
        self.assertEqual(self.obj.status, STATUS_PENDING)

    def test_status_choices(self):
        for k, v in STATUS_CHOICES:
            self.obj.status = k
            self.obj.full_clean()  # no exception raised

    def test_source(self):
        self.assertEqual(self.obj.source, 'source')

    def test_target(self):
        self.assertEqual(self.obj.target, 'target')

    def test_commit_message(self):
        self.assertEqual(self.obj.commit_message, 'commit')

    def test_author(self):
        self.assertEqual(self.obj.author, 'Some Person <some@domain>')

    def test_reviewers(self):
        self.assertEqual(self.obj.reviewers, 'fulano, mengano')

    def test_date_created(self):
        self.assertEqual(self.obj.date_created, self.created)

    def test_date_completed(self):
        self.assertEqual(self.obj.date_completed, None)

    def test_complete(self):
        completed = datetime(2013, 8, 23)
        with patch('locolanderweb.models.datetime') as mock_datetime:
            mock_datetime.utcnow.return_value = completed
            self.obj.complete()

        self.assertEqual(self.obj.status, STATUS_MERGED)
        self.assertEqual(self.obj.date_completed, completed)

    def test_complete_error(self):
        completed = datetime(2013, 8, 23)
        with patch('locolanderweb.models.datetime') as mock_datetime:
            mock_datetime.utcnow.return_value = completed
            self.obj.complete(error=True)

        self.assertEqual(self.obj.status, STATUS_ERROR)
        self.assertEqual(self.obj.date_completed, completed)


class RunLogTestCase(BaseTestCase):

    def setUp(self):
        super(RunLogTestCase, self).setUp()
        self.merge_request = self.factory.make_merge_request()
        self.obj = RunLog.objects.create(
            merge_request=self.merge_request, return_code=0)

    def test_merge_request(self):
        self.assertEqual(self.obj.merge_request, self.merge_request)

    def test_return_code(self):
        self.assertEqual(self.obj.return_code, 0)

    def test_raw(self):
        self.assertEqual(self.obj.raw, '')

    def test_timestamp(self):
        before = datetime.utcnow()
        self.obj = RunLog.objects.create(
            merge_request=self.merge_request, return_code=0)
        after = datetime.utcnow()
        self.assertTrue(before <= self.obj.timestamp)
        self.assertTrue(self.obj.timestamp <= after)

    def test_get_latest_by(self):
        other = RunLog.objects.create(
            merge_request=self.merge_request, return_code=0,
            timestamp=self.obj.timestamp + timedelta(seconds=1))
        self.assertEqual(RunLog.objects.latest(), other)
