"""GitHub service wrapper."""

import re

from github3 import GitHub

from repos import errors


GITHUB_SSH_RE = re.compile(r'^git@github.com:([^/]+)/(.+).git$')
GITHUB_HTTPS_RE = re.compile(r'^https://github.com/([^/]+)/(.+?)(?:.git)?$')


def is_valid_url(url):
    """Return True if the given url is from a GitHub project."""
    return bool(GITHUB_SSH_RE.match(url) or GITHUB_HTTPS_RE.match(url))


class Service(object):

    def __init__(self):
        super(Service, self).__init__()
        # GitHub API access
        self.gh = GitHub()

    def _is_approved_comment(self, repo, comment):
        # this is an 'approved' comment by a commiter
        lines = comment.body.split('\n')
        approved = False
        commit_message = ''
        if repo.is_collaborator(comment.user.login) and len(lines) > 0:
            approved = lines[0].strip().lower() == 'approved'
            commit_message = '\n'.join(lines[1:])

        return approved, commit_message

    def approved_requests_for_project_url(self, project_url):
        repo = None
        match = (GITHUB_SSH_RE.match(project_url) or
                 GITHUB_HTTPS_RE.match(project_url))
        if match:
            repo = self.gh.repository(*match.groups())

        if repo is None:
            raise errors.RepositoryDoesNotExist()

        approved = []
        pulls = repo.iter_pulls('open')
        for pull in pulls:
            comments = list(pull.iter_issue_comments())
            last_comment = comments[-1]
            is_approved, commit_message = self._is_approved_comment(
                repo, last_comment)
            if is_approved:
                approved.append({
                    # XXX: needs UTC processing?
                    'date_created': pull.created_at,
                    ### 'source_repo': self.gh.repository(*pull.base.repo),
                    'source': pull.base.ref,
                    ### 'target_repo': self.gh.repository(*pull.head.repo),
                    'target': pull.head.ref,
                    'reviewers': last_comment.user.login,  # csv
                    'commit_message': commit_message,
                })
        return approved
