"""Launchpad service wrapper."""

from __future__ import unicode_literals

import os
import re

from datetime import datetime

import bzrlib.plugin
from bzrlib.branch import Branch
from launchpadlib.launchpad import Launchpad

from repos import errors


CACHE_DIR = os.path.join(os.path.expanduser('~'), '.launchpadlib', 'cache')
LAUNCHPAD_LP_RE = re.compile(r'^lp:(?:~[^/]+/)?([^/]+)(?:/[^/]+)?$')
LAUNCHPAD_HTTPS_RE = re.compile(
    r'^http(s)?://launchpad.net/(?:~[^/]+/)?([^/]+)$')


def is_valid_url(url):
    """Return True if the given url is from a Launchpad project."""
    return bool(LAUNCHPAD_LP_RE.match(url) or LAUNCHPAD_HTTPS_RE.match(url))


class Service(object):

    def __init__(self):
        super(Service, self).__init__()
        # Launchpad API access
        self.launchpad = Launchpad.login_anonymously(
            'locolander', 'production', CACHE_DIR)
        self.lp_projects = self.launchpad.projects
        # load bzr plugins so that lp: namespace is recognized
        bzrlib.plugin.load_plugins()

    def get_author(self, branch_url):
        branch = Branch.open(branch_url)
        tip = branch.repository.get_revision(
            branch.get_rev_id(branch.revno()))
        return tip.committer

    def approved_requests_for_project_url(self, project_url):
        """Return approved merge proposals."""
        repo = None
        match = (LAUNCHPAD_HTTPS_RE.match(project_url) or
                 LAUNCHPAD_LP_RE.match(project_url))
        if match:
            name = match.groups()[0]
            try:
                repo = self.lp_projects[name]
            except KeyError:
                repo = None

        if repo is None:
            raise errors.RepositoryDoesNotExist()

        approved = []
        proposals = repo.getMergeProposals()
        for mp in proposals:
            if mp.queue_status.lower() == 'approved':
                created = mp.date_created
                approved.append({
                    # XXX: needs UTC processing?
                    'date_created': datetime(created.year, created.month,
                                             created.day),
                    'source': mp.source_branch.bzr_identity,
                    'target': mp.target_branch.bzr_identity,
                    'reviewers': mp.reviewer.name,  # csv
                    'commit_message': mp.commit_message,
                    'author': self.get_author(mp.source_branch.bzr_identity),
                })
        return approved
