from __future__ import unicode_literals

from datetime import datetime
from unittest import TestCase

from mock import Mock, patch

from repos import errors, github


class FakeComment(object):
    """Fake GitHub pull request comment."""

    def __init__(self, status=None, is_collaborator=False):
        self.status = status
        self.body = 'Whatever'
        username = 'nobody'
        if is_collaborator:
            username = 'collaborator'
        self.user = Mock(login=username)
        if status == 'approved':
            self.body = 'approved\ncommit msg'


class FakePullRequest(object):
    """Fake GitHub pull request."""

    def __init__(self, status=None, is_approved=False):
        self.is_approved = is_approved
        self.status = status
        self.created_at = datetime.now()
        self.base = FakeRepo()
        self.head = FakeRepo()

    def iter_issue_comments(self):
        is_collaborator = self.is_approved
        return [FakeComment(),
                FakeComment(status=self.status,
                            is_collaborator=is_collaborator)]


class FakeRepo(object):
    """Fake GitHub project repo."""

    def __init__(self, is_approved=False):
        self.is_approved = is_approved
        self.ref = 'master'
        self.status = 'no-approved'
        if is_approved:
            self.status = 'approved'

    def iter_pulls(self, state):
        if self.is_approved:
            pulls = [FakePullRequest(),
                     FakePullRequest(status='approved', is_approved=True)]
        else:
            pulls = [FakePullRequest()]
        return pulls

    def is_collaborator(self, username):
        return username == 'collaborator'


class GitHubMock(object):
    """Fake GitHub API wrapper."""

    def __init__(self):
        self.data = {}
        repos = {
            'test-repo': FakeRepo(is_approved=True),
            'another-repo': FakeRepo(),
        }
        self.data['test-user'] = repos

    def repository(self, username, repo_name):
        return self.data.get(username, {}).get(repo_name, None)


class GitHubServiceTestCase(TestCase):

    def setUp(self):
        super(GitHubServiceTestCase, self).setUp()
        self.mock_gh = patch('repos.github.GitHub', GitHubMock)
        self.mock_gh.start()
        self.addCleanup(self.mock_gh.stop)
        self.gh = github.Service()

    def test_project_not_match(self):
        with self.assertRaises(errors.RepositoryDoesNotExist):
            self.gh.approved_requests_for_project_url('test/not-match')

    def test_username_not_found(self):
        with self.assertRaises(errors.RepositoryDoesNotExist):
            self.gh.approved_requests_for_project_url(
                'git@github.com:test-user/not-found')

    def test_project_not_found(self):
        with self.assertRaises(errors.RepositoryDoesNotExist):
            self.gh.approved_requests_for_project_url(
                'git@github.com:test/not-found')

    def test_valid_project_without_approved_mp(self):
        mps = self.gh.approved_requests_for_project_url(
            'git@github.com:test-user/another-repo.git')
        self.assertEqual(len(mps), 0)

    def test_valid_project_return_approved_mp(self):
        mps = self.gh.approved_requests_for_project_url(
            'git@github.com:test-user/test-repo.git')
        self.assertEqual(len(mps), 1)
        mp = mps[0]
        self.assertIsInstance(mp, dict)

        expected_keys = ['date_created', 'source', 'target', 'reviewers',
                         'commit_message']
        for key in expected_keys:
            self.assertIn(key, mp)
        self.assertEqual(mp['commit_message'], 'commit msg')
