"""Nessita Syntax to Docker File.

Example:

    precise:
        apt:
            apache2: 3.3-4
            bzr:
        pip:
            foobar:
    metadata:
        test_script: ./test


will be converted to

    from locolander:precise
    run apt-get -q -y install apache2=3.3-4
    run apt-get -q -y install bzr
    run pip install --download=/tmp/pipcache --no-install foobar
    run ip link set dev eth0 down
    run pip install --find-links=file:///tmp/pipcache --no-index foobar

and also the system will use "./test" as script
"""

import os.path

import yaml

# the conversion between our nice base names and those that are
# needed by docker
BASE_TRANSLATIONS = {
    'precise': 'locolander:precise',
}


def _get_pip_packages(items):
    if isinstance(items, basestring):
        return items
    packages = []
    for name, ver in sorted(items.items()):
        if ver is None:
            p = name
        else:
            p = "%s==%s" % (name, ver)
        packages.append(p)
    return ' '.join(packages)


def _get_base(config, **params):
    """Process the system base stuff."""
    # support only one base for now
    if len(config) > 1:
        raise NotImplementedError("Too many base machines.")
    base = config.keys()[0]
    return ["from " + BASE_TRANSLATIONS[base]]


def _get_code(config, **params):
    return ['run mkdir -p {target_path}'.format(**params),
            'add . {target_path}'.format(**params),
            'run chown -R locolander.locolander {target_path}'.format(
                **params)]


def _get_depends_prev(config, **params):
    """All the dependencies before securing machine."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # apt
    items = config.get('apt', [])
    if items:
        packages = []
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s=%s" % (name, ver)
            packages.append(p)
        params['packages'] = ' '.join(packages)
        dependencies.append('run apt-get -q -y install {packages}'.format(**params))

    # pip
    items = config.get('pip', [])
    if items:
        packages = _get_pip_packages(items)
        pip_cache_dir = params['pip_cache_dir']
        dependencies.append(
            'run cd {target_path} && pip install --download={pip_cache_dir} '
            '--no-install {packages}'.format(
                pip_cache_dir=pip_cache_dir, packages=packages,
                target_path=params['target_path']))

    return dependencies

def _get_secure(config, **params):
    """All that needs to be done to secure the machine."""
    return ["run ip link set dev eth0 down"]

def _get_depends_post(config, **params):
    """All the dependencies stuff after the machine was secured."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # pip
    items = config.get('pip', [])
    if items:
        packages = _get_pip_packages(items)
        pip_cache_dir = params['pip_cache_dir']
        dependencies.append(
            'run cd {target_path} && pip install '
            '--find-links=file://{pip_cache_dir} --no-index '
            '{packages}'.format(pip_cache_dir=pip_cache_dir,
                                packages=packages,
                                target_path=params['target_path']))

    return dependencies


def _get_rest(config, **params):
    """The final stuff."""
    return ['cmd cd {target_path} && timeout 300 {test_cmd}'.format(**params)]


def parse(config_text, pip_cache_dir, project):
    """Convert Nessita Syntax and return the text for Docker."""
    config = yaml.load(config_text)
    metadata = config.pop("metadata")

    params = {
        'test_cmd': metadata['test_script'],
        'target_path': os.path.join('/srv/locolander', project),
        'pip_cache_dir': pip_cache_dir,
    }

    # the order for calling these functions is VERY important
    functions = [
        _get_base,
        _get_code,
        _get_depends_prev,
        _get_secure,
        _get_depends_post,
        _get_rest,
    ]
    data = []
    for func in functions:
        items = func(config, **params)
        data.extend(items)
    docker = "\n".join(data)

    return docker


if __name__ == '__main__':
    import sys

    project = sys.argv[1]
    infile, outfile = sys.argv[2:]

    config = open(infile, 'r').read()
    data = parse(config, '/var/cache/locolander', project)
    with open(outfile, 'w') as dockerfile:
        dockerfile.write(data + '\n')
