"""Tests for the ns2df convertor."""

import unittest

import ns2df

class DependenciesTestCase(unittest.TestCase):
    """Tests for the dependencies transformer."""

    def setUp(self):
        super(DependenciesTestCase, self).setUp()
        self.params = {
            'pip_cache_dir': '/tmp/pipcache',
            'target_path': '/srv/locolander/project',
        }

    def test_no_dependencies(self):
        config = dict(somebase={})
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [])
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [])

    def test_apt_one(self):
        config = dict(somebase={
            'apt': {'bzr': None}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, ["run apt-get -q -y install bzr"])

    def test_apt_several(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': None}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            "run apt-get -q -y install apache2 bzr",
        ])

    def test_apt_versions(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            "run apt-get -q -y install apache2=3.3-4 bzr",
        ])

    def test_pip_prev_one(self):
        config = dict(somebase={
            'pip': {'foo': None}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            "run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install foo",
        ])

    def test_pip_prev_several(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': None}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            "run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install bar foo",
        ])

    def test_pip_prev_versions(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': '2.1'}
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install "
             "bar==2.1 foo"),
        ])

    def test_pip_prev_requirements(self):
        config = dict(somebase={
            'pip': '-r requirements.txt'
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install "
             "-r requirements.txt"),
        ])

    def test_pip_post_one(self):
        config = dict(somebase={
            'pip': {'foo': None}
        })
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [
            "run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache --no-index foo"
        ])

    def test_pip_post_several(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': None}
        })
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache --no-index "
             "bar foo"),
        ])

    def test_pip_post_versions(self):
        config = dict(somebase={
            'pip': {'foo': None, 'bar': '2.1'}
        })
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache --no-index "
             "bar==2.1 foo"),
        ])

    def test_pip_post_requirements(self):
        config = dict(somebase={
            'pip': '-r requirements.txt'
        })
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache --no-index "
             "-r requirements.txt"),
        ])

    def test_mixed_prev(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'},
            'pip': {'foo': None, 'bar': None},
        })
        res = ns2df._get_depends_prev(config, **self.params)
        self.assertEqual(res, [
            "run apt-get -q -y install apache2=3.3-4 bzr",
            "run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install bar foo",
        ])

    def test_mixed_post(self):
        config = dict(somebase={
            'apt': {'bzr': None, 'apache2': '3.3-4'},
            'pip': {'foo': None, 'bar': None},
        })
        res = ns2df._get_depends_post(config, **self.params)
        self.assertEqual(res, [
            ("run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache --no-index "
             "bar foo"),
        ])


class BoilerplateTestCase(unittest.TestCase):
    """Tests for the setup and the rest of commands."""

    def test_one_base(self):
        config = dict(precise={})
        res = ns2df._get_base(config)
        self.assertEqual(res, ["from locolander:precise"])

    def test_several_bases(self):
        config = dict(base1={}, base2={})
        self.assertRaises(NotImplementedError, ns2df._get_base, config)

    def test_secure_actions(self):
        res = ns2df._get_secure({})
        self.assertEqual(res, ["run ip link set dev eth0 down"])

    def test_rest(self):
        res = ns2df._get_rest({},
                              target_path='/srv/locolander',
                              test_cmd='./run_tests.sh')
        self.assertEqual(res, ["cmd cd /srv/locolander && timeout 300 ./run_tests.sh"])

    def test_all_mixed(self):
        config_text = """
            precise:
                apt:
                    apache2: 3.3-4
                    bzr:
                pip:
                    foobar:
            metadata:
                test_script: foo
        """
        docker = ns2df.parse(config_text, "/tmp/pipcache", 'project')
        self.assertEqual(docker,
            "from locolander:precise\n"
            "run mkdir -p /srv/locolander/project\n"
            "add . /srv/locolander/project\n"
            "run chown -R locolander.locolander /srv/locolander/project\n"
            "run apt-get -q -y install apache2=3.3-4 bzr\n"
            "run cd /srv/locolander/project && pip install --download=/tmp/pipcache --no-install foobar\n"
            "run ip link set dev eth0 down\n"
            "run cd /srv/locolander/project && pip install --find-links=file:///tmp/pipcache "
                "--no-index foobar\n"
            "cmd cd /srv/locolander/project && timeout 300 foo"
        )
